--- @module molde
local molde = {}

--- Module version 0.1.0
molde.VERSION = "0.1.0"

--- Long string bracket level.
--
-- It is used as `string.rep('=', molde.string_bracket_level)` to create the
-- necessary string delimiters for literals.
molde.string_bracket_level = 1

--- Parse errors that can occur in a template
molde.errors = {
	PegError = nil,	             -- When PEG can't parse
	ClosingValueError = nil,     -- There is no closing `"}}"` to a value
	ClosingStatementError = nil, -- There is no closing `"%}"` to a statement
	EmptyValueError = nil,       -- There is no content after value opening `"{{"`
	EmptyStatementError = nil,   -- There is no content after statement opening `"{%"`
}

--- Parse a template, returning a table with the matched contents.
--
-- The parser tags the contents as:
--
-- + Literal: text that should be just copied to result
-- + Value: a value to be substituted using Lua, usually a variable; it will be
--   stringified using `tostring`
-- + Statement: one or more Lua statements that will be copied directly into the
--   compiled function
--
-- Results are in the format `{['literal' | 'value' | 'statement'] = <captured value>}`
--
-- @param template Template string to be parsed
--
-- @return[1] Table with results
-- @return[2] `nil`
-- @return[2] Parse error
--
-- @usage
--   local results = molde.parse([[literal {{ value }} {% statement %}]])
--   --[[
--   -- results: {
--   --     {literal = "literal "},
--   --     {value = " value "},
--   --     {literal = " "},
--   --     {statement = " statement "}
--   -- }
--   --]]
function molde.parse(template) end

--- Compiles a table with contents to Lua code that generates the (hopefully)
-- desired result.
--
-- The code generated for literals use Lua's long strings, with a default level
-- of 1. This level is taken from `molde.string_bracket_level`, and can be changed
-- if your literal may contain the string `"]=]"`, for example.
--
-- @param template Template string to be compiled
--
-- @treturn[1] string Generated code
-- @return[2] `nil`
-- @return[2] Parse error
function molde.compile(template) end


--- Compiles the template, returning a closure that executes the substitution.
--
-- The returned function behaves as described in `__process_template_function`.
--
-- @param template Template string
-- @treturn[1] function Template processor
-- @return[2] `nil`
-- @return[2] Parse error
--
-- @usage
--   hello_template = molde.load([[Hello {{ name or "world" }}]])
--   print(hello_template()) -- "Hello world"
function molde.load(template) end


--- Same as `molde.load`, but loads the template from a file.
--
-- Every caveat for `molde.load` applies.
--
-- @see molde.load
--
-- @param template_file Template file path
-- @treturn[1] function Template processor
-- @return[2] `nil`
-- @return[2] File open error
-- @return[3] `nil`
-- @return[3] Parse error
--
-- @usage
--   hello_template = molde.loadfile("hello_template_file")
--   print(hello_template()) -- "Hello world"
function molde.loadfile(template_file) end

--- This is the prototype of the function returned by `molde.load` and
-- `molde.loadfile`.
--
-- @raise When the generated code is invalid
--
-- @param[opt=_ENV] env The used environment
-- @treturn string Processed template
function __process_template_function(env) end
